@@ -1,5 +1,5 @@
 // Symphonia
-// Copyright (c) 2019 The Project Symphonia Developers.
+// Copyright (c) 2019-2022 The Project Symphonia Developers.
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,142 +13,239 @@ use std::default::Default;
 use std::fmt;
 
 use crate::audio::{AudioBufferRef, Channels, Layout};
-use crate::errors::{Result, unsupported_error};
+use crate::errors::{unsupported_error, Result};
 use crate::formats::Packet;
 use crate::sample::SampleFormat;
 use crate::units::TimeBase;
 
-/// A `CodecType` is a unique identifier used to identify a specific codec. `CodecType` is mainly
-/// used for matching a format's stream to a specific `Decoder`. Decoders advertisting support for a
-/// specific `CodecType` should be interchangeable in regards to their ability to consume packets
-/// from a packet stream. This means that while support for codec features and quality may differ,
-/// all Decoders will identically advance the packet stream.
-#[derive(Copy, Clone, PartialEq, Eq, Hash)]
+/// A `CodecType` is a unique identifier used to identify a specific codec.
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
 pub struct CodecType(u32);
 
-/// Null decoder, simply discards all data.
-pub const CODEC_TYPE_NULL: CodecType             = CodecType(0x0);
+/// Declares a new `CodecType` given a character code. A character code is an ASCII string
+/// containing a maximum of 5 alphanumeric characters.
+///
+/// Note: Due to the current limitations of const fn, this function is not able to panic on error.
+/// Therefore, if the character code contains an invalid character, then the character is dropped.
+/// Additionally, any extra characters will be truncated.
+pub const fn decl_codec_type(cc: &[u8]) -> CodecType {
+    /// Map alphanumeric ASCII characters into a 6-bit code.
+    const fn map_ascii_to_bits(cc: u8) -> u32 {
+        // The mapping is defined as:
+        //  b'0'..=b'9' maps to  1..=10
+        //  b'a'..=b'z' maps to 11..=36
+        //  b'A'..=b'Z' maps to 37..=62
+        if cc.is_ascii_digit() {
+            1 + (cc - b'0') as u32
+        }
+        else if cc.is_ascii_lowercase() {
+            11 + (cc - b'a') as u32
+        }
+        else if cc.is_ascii_uppercase() {
+            37 + (cc - b'A') as u32
+        }
+        else {
+            0
+        }
+    }
+
+    // TODO: assert!(cc.len() <= 5);
+
+    // The upper-bit indicates the user codec namespace.
+    let mut id = 0x8000_0000;
+
+    let mut i = 0;
+    let mut j = 0;
+
+    while i < cc.len() && j < 5 {
+        // TODO: When const panic is stabilized, assert that the character is alphanumeric to
+        // generate an error rather than silently dropping invalid characters.
+        // assert!(cc[i].is_ascii_alphanumeric());
+
+        // Pack the ASCII characters into the allocated 30 bits (6 bits per character) in MSb order.
+        if cc[i].is_ascii_alphanumeric() {
+            id |= map_ascii_to_bits(cc[i]) << (24 - (6 * j));
+            j += 1;
+        }
+        i += 1;
+    }
+
+    CodecType(id)
+}
+
+impl fmt::Display for CodecType {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "{:#x}", self.0)
+    }
+}
+
+/// Null codec
+pub const CODEC_TYPE_NULL: CodecType = CodecType(0x0);
 
 // Uncompressed PCM audio codecs
 //------------------------------
 
 /// PCM signed 32-bit little-endian interleaved
-pub const CODEC_TYPE_PCM_S32LE: CodecType        = CodecType(0x100);
+pub const CODEC_TYPE_PCM_S32LE: CodecType = CodecType(0x100);
 /// PCM signed 32-bit little-endian planar
 pub const CODEC_TYPE_PCM_S32LE_PLANAR: CodecType = CodecType(0x101);
 /// PCM signed 32-bit big-endian interleaved
-pub const CODEC_TYPE_PCM_S32BE: CodecType        = CodecType(0x102);
+pub const CODEC_TYPE_PCM_S32BE: CodecType = CodecType(0x102);
 /// PCM signed 32-bit big-endian planar
 pub const CODEC_TYPE_PCM_S32BE_PLANAR: CodecType = CodecType(0x103);
 /// PCM signed 24-bit little-endian interleaved
-pub const CODEC_TYPE_PCM_S24LE: CodecType        = CodecType(0x104);
+pub const CODEC_TYPE_PCM_S24LE: CodecType = CodecType(0x104);
 /// PCM signed 24-bit little-endian planar
 pub const CODEC_TYPE_PCM_S24LE_PLANAR: CodecType = CodecType(0x105);
 /// PCM signed 24-bit big-endian interleaved
-pub const CODEC_TYPE_PCM_S24BE: CodecType        = CodecType(0x106);
+pub const CODEC_TYPE_PCM_S24BE: CodecType = CodecType(0x106);
 /// PCM signed 24-bit big-endian planar
 pub const CODEC_TYPE_PCM_S24BE_PLANAR: CodecType = CodecType(0x107);
 /// PCM signed 16-bit little-endian interleaved
-pub const CODEC_TYPE_PCM_S16LE: CodecType        = CodecType(0x108);
+pub const CODEC_TYPE_PCM_S16LE: CodecType = CodecType(0x108);
 /// PCM signed 16-bit little-endian planar
 pub const CODEC_TYPE_PCM_S16LE_PLANAR: CodecType = CodecType(0x109);
 /// PCM signed 16-bit big-endian interleaved
-pub const CODEC_TYPE_PCM_S16BE: CodecType        = CodecType(0x10a);
+pub const CODEC_TYPE_PCM_S16BE: CodecType = CodecType(0x10a);
 /// PCM signed 16-bit big-endian planar
 pub const CODEC_TYPE_PCM_S16BE_PLANAR: CodecType = CodecType(0x10b);
 /// PCM signed 8-bit interleaved
-pub const CODEC_TYPE_PCM_S8: CodecType           = CodecType(0x10c);
+pub const CODEC_TYPE_PCM_S8: CodecType = CodecType(0x10c);
 /// PCM signed 8-bit planar
-pub const CODEC_TYPE_PCM_S8_PLANAR: CodecType    = CodecType(0x10d);
+pub const CODEC_TYPE_PCM_S8_PLANAR: CodecType = CodecType(0x10d);
 /// PCM unsigned 32-bit little-endian interleaved
-pub const CODEC_TYPE_PCM_U32LE: CodecType        = CodecType(0x10e);
+pub const CODEC_TYPE_PCM_U32LE: CodecType = CodecType(0x10e);
 /// PCM unsigned 32-bit little-endian planar
 pub const CODEC_TYPE_PCM_U32LE_PLANAR: CodecType = CodecType(0x10f);
 /// PCM unsigned 32-bit big-endian interleaved
-pub const CODEC_TYPE_PCM_U32BE: CodecType        = CodecType(0x110);
+pub const CODEC_TYPE_PCM_U32BE: CodecType = CodecType(0x110);
 /// PCM unsigned 32-bit big-endian planar
 pub const CODEC_TYPE_PCM_U32BE_PLANAR: CodecType = CodecType(0x111);
 /// PCM unsigned 24-bit little-endian interleaved
-pub const CODEC_TYPE_PCM_U24LE: CodecType        = CodecType(0x112);
+pub const CODEC_TYPE_PCM_U24LE: CodecType = CodecType(0x112);
 /// PCM unsigned 24-bit little-endian planar
 pub const CODEC_TYPE_PCM_U24LE_PLANAR: CodecType = CodecType(0x113);
 /// PCM unsigned 24-bit big-endian interleaved
-pub const CODEC_TYPE_PCM_U24BE: CodecType        = CodecType(0x114);
+pub const CODEC_TYPE_PCM_U24BE: CodecType = CodecType(0x114);
 /// PCM unsigned 24-bit big-endian planar
 pub const CODEC_TYPE_PCM_U24BE_PLANAR: CodecType = CodecType(0x115);
 /// PCM unsigned 16-bit little-endian interleaved
-pub const CODEC_TYPE_PCM_U16LE: CodecType        = CodecType(0x116);
+pub const CODEC_TYPE_PCM_U16LE: CodecType = CodecType(0x116);
 /// PCM unsigned 16-bit little-endian planar
 pub const CODEC_TYPE_PCM_U16LE_PLANAR: CodecType = CodecType(0x117);
 /// PCM unsigned 16-bit big-endian interleaved
-pub const CODEC_TYPE_PCM_U16BE: CodecType        = CodecType(0x118);
+pub const CODEC_TYPE_PCM_U16BE: CodecType = CodecType(0x118);
 /// PCM unsigned 16-bit big-endian planar
 pub const CODEC_TYPE_PCM_U16BE_PLANAR: CodecType = CodecType(0x119);
 /// PCM unsigned 8-bit interleaved
-pub const CODEC_TYPE_PCM_U8: CodecType           = CodecType(0x11a);
+pub const CODEC_TYPE_PCM_U8: CodecType = CodecType(0x11a);
 /// PCM unsigned 8-bit planar
-pub const CODEC_TYPE_PCM_U8_PLANAR: CodecType    = CodecType(0x11b);
+pub const CODEC_TYPE_PCM_U8_PLANAR: CodecType = CodecType(0x11b);
 /// PCM 32-bit little-endian floating point interleaved
-pub const CODEC_TYPE_PCM_F32LE: CodecType        = CodecType(0x11c);
+pub const CODEC_TYPE_PCM_F32LE: CodecType = CodecType(0x11c);
 /// PCM 32-bit little-endian floating point planar
 pub const CODEC_TYPE_PCM_F32LE_PLANAR: CodecType = CodecType(0x11d);
 /// PCM 32-bit big-endian floating point interleaved
-pub const CODEC_TYPE_PCM_F32BE: CodecType        = CodecType(0x11e);
+pub const CODEC_TYPE_PCM_F32BE: CodecType = CodecType(0x11e);
 /// PCM 32-bit big-endian floating point planar
 pub const CODEC_TYPE_PCM_F32BE_PLANAR: CodecType = CodecType(0x11f);
 /// PCM 64-bit little-endian floating point interleaved
-pub const CODEC_TYPE_PCM_F64LE: CodecType        = CodecType(0x120);
+pub const CODEC_TYPE_PCM_F64LE: CodecType = CodecType(0x120);
 /// PCM 64-bit little-endian floating point planar
 pub const CODEC_TYPE_PCM_F64LE_PLANAR: CodecType = CodecType(0x121);
 /// PCM 64-bit big-endian floating point interleaved
-pub const CODEC_TYPE_PCM_F64BE: CodecType        = CodecType(0x122);
+pub const CODEC_TYPE_PCM_F64BE: CodecType = CodecType(0x122);
 /// PCM 64-bit big-endian floating point planar
 pub const CODEC_TYPE_PCM_F64BE_PLANAR: CodecType = CodecType(0x123);
-/// PCM A-law
-pub const CODEC_TYPE_PCM_ALAW: CodecType         = CodecType(0x124);
-/// PCM Mu-law
-pub const CODEC_TYPE_PCM_MULAW: CodecType        = CodecType(0x125);
+/// PCM A-law (G.711)
+pub const CODEC_TYPE_PCM_ALAW: CodecType = CodecType(0x124);
+/// PCM Mu-law (G.711)
+pub const CODEC_TYPE_PCM_MULAW: CodecType = CodecType(0x125);
+
+// ADPCM audio codecs
+//-------------------
+
+/// G.722 ADPCM
+pub const CODEC_TYPE_ADPCM_G722: CodecType = CodecType(0x200);
+/// G.726 ADPCM
+pub const CODEC_TYPE_ADPCM_G726: CodecType = CodecType(0x201);
+/// G.726 ADPCM little-endian
+pub const CODEC_TYPE_ADPCM_G726LE: CodecType = CodecType(0x202);
+/// Microsoft ADPCM
+pub const CODEC_TYPE_ADPCM_MS: CodecType = CodecType(0x203);
+/// ADPCM IMA WAV
+pub const CODEC_TYPE_ADPCM_IMA_WAV: CodecType = CodecType(0x204);
+/// ADPCM IMA QuickTime
+pub const CODEC_TYPE_ADPCM_IMA_QT: CodecType = CodecType(0x205);
 
 // Compressed lossy audio codecs
 //------------------------------
 
 /// Vorbis
-pub const CODEC_TYPE_VORBIS: CodecType           = CodecType(0x1000);
+pub const CODEC_TYPE_VORBIS: CodecType = CodecType(0x1000);
 /// MPEG Layer 1 (MP1)
-pub const CODEC_TYPE_MP1: CodecType              = CodecType(0x1001);
+pub const CODEC_TYPE_MP1: CodecType = CodecType(0x1001);
 /// MPEG Layer 2 (MP2)
-pub const CODEC_TYPE_MP2: CodecType              = CodecType(0x1002);
+pub const CODEC_TYPE_MP2: CodecType = CodecType(0x1002);
 /// MPEG Layer 3 (MP3)
-pub const CODEC_TYPE_MP3: CodecType              = CodecType(0x1003);
+pub const CODEC_TYPE_MP3: CodecType = CodecType(0x1003);
 /// Advanced Audio Coding (AAC)
-pub const CODEC_TYPE_AAC: CodecType              = CodecType(0x1004);
+pub const CODEC_TYPE_AAC: CodecType = CodecType(0x1004);
 /// Opus
-pub const CODEC_TYPE_OPUS: CodecType             = CodecType(0x1005);
+pub const CODEC_TYPE_OPUS: CodecType = CodecType(0x1005);
+/// Speex
+pub const CODEC_TYPE_SPEEX: CodecType = CodecType(0x1006);
 /// Musepack
-pub const CODEC_TYPE_MUSEPACK: CodecType         = CodecType(0x1006);
+pub const CODEC_TYPE_MUSEPACK: CodecType = CodecType(0x1007);
+/// Adaptive Transform Acoustic Coding (ATRAC1)
+pub const CODEC_TYPE_ATRAC1: CodecType = CodecType(0x1008);
+/// Adaptive Transform Acoustic Coding 3 (ATRAC3)
+pub const CODEC_TYPE_ATRAC3: CodecType = CodecType(0x1009);
+/// Adaptive Transform Acoustic Coding 3+ (ATRAC3+)
+pub const CODEC_TYPE_ATRAC3PLUS: CodecType = CodecType(0x100a);
+/// Adaptive Transform Acoustic Coding 9 (ATRAC9)
+pub const CODEC_TYPE_ATRAC9: CodecType = CodecType(0x100b);
+/// AC-3, E-AC-3, Dolby Digital (ATSC A/52)
+pub const CODEC_TYPE_EAC3: CodecType = CodecType(0x100c);
+/// Dolby AC-4 (ETSI TS 103 190)
+pub const CODEC_TYPE_AC4: CodecType = CodecType(0x100d);
+/// DTS Coherent Acoustics (DCA/DTS)
+pub const CODEC_TYPE_DCA: CodecType = CodecType(0x100e);
+/// Windows Media Audio
+pub const CODEC_TYPE_WMA: CodecType = CodecType(0x100f);
 
 // Compressed lossless audio codecs
 //---------------------------------
 
 /// Free Lossless Audio Codec (FLAC)
-pub const CODEC_TYPE_FLAC: CodecType             = CodecType(0x2000);
+pub const CODEC_TYPE_FLAC: CodecType = CodecType(0x2000);
 /// WavPack
-pub const CODEC_TYPE_WAVPACK: CodecType          = CodecType(0x2001);
+pub const CODEC_TYPE_WAVPACK: CodecType = CodecType(0x2001);
 /// Monkey's Audio (APE)
-pub const CODEC_TYPE_MONKEYS_AUDIO: CodecType    = CodecType(0x2002);
+pub const CODEC_TYPE_MONKEYS_AUDIO: CodecType = CodecType(0x2002);
 /// Apple Lossless Audio Codec (ALAC)
-pub const CODEC_TYPE_ALAC: CodecType             = CodecType(0x2003);
+pub const CODEC_TYPE_ALAC: CodecType = CodecType(0x2003);
+/// True Audio (TTA)
+pub const CODEC_TYPE_TTA: CodecType = CodecType(0x2004);
 
-
-impl fmt::Display for CodecType {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "{}", self.0)
-    }
+/// A method and expected value to perform verification on the decoded audio.
+#[derive(Copy, Clone, Debug)]
+pub enum VerificationCheck {
+    /// CRC8 of interleaved PCM audio samples.
+    Crc8(u8),
+    /// CRC16 of interleaved PCM audio samples.
+    Crc16([u8; 2]),
+    /// CRC32 of interleaved PCM audio samples.
+    Crc32([u8; 4]),
+    /// MD5 of interleaved PCM audio samples.
+    Md5([u8; 16]),
+    /// Codec defined, up-to 16-byte code.
+    Other([u8; 16]),
 }
 
 /// Codec parameters stored in a container format's headers and metadata may be passed to a codec
 /// using the `CodecParameters` structure.
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct CodecParameters {
     /// The codec type.
     pub codec: CodecType,
@@ -162,6 +259,9 @@ pub struct CodecParameters {
     /// The length of the encoded stream in number of frames.
     pub n_frames: Option<u64>,
 
+    /// The timestamp of the first frame.
+    pub start_ts: u64,
+
     /// The sample format of an audio sample.
     pub sample_format: Option<SampleFormat>,
 
@@ -177,13 +277,12 @@ pub struct CodecParameters {
     /// The channel layout.
     pub channel_layout: Option<Layout>,
 
-    /// The number of leading frames inserted by the encoder for padding that should be skipped
-    /// during playback.
-    pub leading_padding: Option<u32>,
+    /// The number of leading frames inserted by the encoder that should be skipped during playback.
+    pub delay: Option<u32>,
 
     /// The number of trailing frames inserted by the encoder for padding that should be skipped
     /// during playback.
-    pub trailing_padding: Option<u32>,
+    pub padding: Option<u32>,
 
     /// The maximum number of frames a packet will contain.
     pub max_frames_per_packet: Option<u64>,
@@ -191,6 +290,9 @@ pub struct CodecParameters {
     /// The demuxer guarantees packet data integrity.
     pub packet_data_integrity: bool,
 
+    /// A method and expected value that may be used to perform verification on the decoded audio.
+    pub verification_check: Option<VerificationCheck>,
+
     /// Extra data (defined by the codec).
     pub extra_data: Option<Box<[u8]>>,
 }
@@ -202,15 +304,17 @@ impl CodecParameters {
             sample_rate: None,
             time_base: None,
             n_frames: None,
+            start_ts: 0,
             sample_format: None,
             bits_per_sample: None,
             bits_per_coded_sample: None,
             channels: None,
             channel_layout: None,
-            leading_padding: None,
-            trailing_padding: None,
+            delay: None,
+            padding: None,
             max_frames_per_packet: None,
             packet_data_integrity: false,
+            verification_check: None,
             extra_data: None,
         }
     }
@@ -239,6 +343,12 @@ impl CodecParameters {
         self
     }
 
+    /// Provide the timestamp of the first frame.
+    pub fn with_start_ts(&mut self, start_ts: u64) -> &mut Self {
+        self.start_ts = start_ts;
+        self
+    }
+
     /// Provide the codec's decoded audio sample format.
     pub fn with_sample_format(&mut self, sample_format: SampleFormat) -> &mut Self {
         self.sample_format = Some(sample_format);
@@ -269,15 +379,15 @@ impl CodecParameters {
         self
     }
 
-    /// Provide the number of leading frames to discard.
-    pub fn with_leading_padding(&mut self, padding: u32) -> &mut Self {
-        self.leading_padding = Some(padding);
+    /// Provide the number of delay frames.
+    pub fn with_delay(&mut self, delay: u32) -> &mut Self {
+        self.delay = Some(delay);
         self
     }
 
-    /// Provide the number of trailing frames to discard.
-    pub fn with_trailing_padding(&mut self, padding: u32) -> &mut Self {
-        self.trailing_padding = Some(padding);
+    /// Provide the number of padding frames.
+    pub fn with_padding(&mut self, padding: u32) -> &mut Self {
+        self.padding = Some(padding);
         self
     }
 
@@ -299,26 +409,38 @@ impl CodecParameters {
         self
     }
 
+    /// Provide a verification code of the final decoded audio.
+    pub fn with_verification_code(&mut self, code: VerificationCheck) -> &mut Self {
+        self.verification_check = Some(code);
+        self
+    }
+}
+
+impl Default for CodecParameters {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+/// `FinalizeResult` contains optional information that can only be found, calculated, or
+/// determined after decoding is complete.
+#[derive(Copy, Clone, Debug, Default)]
+pub struct FinalizeResult {
+    /// If verification is enabled and supported by the decoder, provides the verification result
+    /// if available.
+    pub verify_ok: Option<bool>,
 }
 
 /// `DecoderOptions` is a common set of options that all decoders use.
+#[derive(Copy, Clone, Debug, Default)]
 pub struct DecoderOptions {
     /// The decoded audio should be verified if possible during the decode process.
     pub verify: bool,
 }
 
-impl Default for DecoderOptions {
-    fn default() -> Self {
-        DecoderOptions {
-            verify: false,
-        }
-    }
-}
-
 /// A `Decoder` implements a codec's decode algorithm. It consumes `Packet`s and produces
 /// `AudioBuffer`s.
-pub trait Decoder: Send {
-
+pub trait Decoder: Send + Sync {
     /// Attempts to instantiates a `Decoder` using the provided `CodecParameters`.
     fn try_new(params: &CodecParameters, options: &DecoderOptions) -> Result<Self>
     where
@@ -329,15 +451,39 @@ pub trait Decoder: Send {
     where
         Self: Sized;
 
-    /// Gets a reference to parameters the `Decoder` was instantiated with.
+    /// Reset the `Decoder`.
+    ///
+    /// A decoder must be reset when the next packet is discontinuous with respect to the last
+    /// decoded packet. Most notably, this occurs after a seek.
+    ///
+    /// For codecs that do a lot of pre-computation, reset should only reset the absolute minimum
+    /// amount of state.
+    fn reset(&mut self);
+
+    /// Gets a reference to an updated set of `CodecParameters` based on the parameters the
+    /// `Decoder` was instantiated with.
     fn codec_params(&self) -> &CodecParameters;
 
     /// Decodes a `Packet` of audio data and returns a copy-on-write generic (untyped) audio buffer
     /// of the decoded audio.
+    ///
+    /// If a `DecodeError` or `IoError` is returned, the packet is undecodeable and should be
+    /// discarded. Decoding may be continued with the next packet. If `ResetRequired` is returned,
+    /// consumers of the decoded audio data should expect the duration and `SignalSpec` of the
+    /// decoded audio buffer to change. All other errors are unrecoverable.
+    ///
+    /// Implementors of decoders *must* `clear` the internal buffer if an error occurs.
     fn decode(&mut self, packet: &Packet) -> Result<AudioBufferRef>;
 
-    /// Closes a decoder.
-    fn close(&mut self);
+    /// Optionally, obtain post-decode information such as the verification status.
+    fn finalize(&mut self) -> FinalizeResult;
+
+    /// Allows read access to the internal audio buffer.
+    ///
+    /// After a successful call to `decode`, this will contain the audio content of the last decoded
+    /// `Packet`. If the last call to `decode` resulted in an error, then implementors *must* ensure
+    /// the returned audio buffer has zero length.
+    fn last_decoded(&self) -> AudioBufferRef;
 }
 
 /// A `CodecDescriptor` stores a description of a single logical codec. Common information such as
@@ -365,9 +511,7 @@ pub struct CodecRegistry {
 impl CodecRegistry {
     /// Instantiate a new `CodecRegistry`.
     pub fn new() -> Self {
-        CodecRegistry {
-            codecs: HashMap::new(),
-        }
+        CodecRegistry { codecs: HashMap::new() }
     }
 
     /// Gets the `CodecDescriptor` for a registered codec.
@@ -379,7 +523,7 @@ impl CodecRegistry {
     /// by another `Decoder` it will be replaced within the registry.
     pub fn register_all<D: Decoder>(&mut self) {
         for descriptor in D::supported_codecs() {
-            self.register(&descriptor);
+            self.register(descriptor);
         }
     }
 
@@ -393,18 +537,26 @@ impl CodecRegistry {
     /// instantiated with the provided `CodecParameters` and returned. If a `Decoder` could not be
     /// found, or the `CodecParameters` are either insufficient or invalid for the `Decoder`, an
     /// error will be returned.
-    pub fn make(&self, params: &CodecParameters, options: &DecoderOptions)
-        -> Result<Box<dyn Decoder>> {
-
+    pub fn make(
+        &self,
+        params: &CodecParameters,
+        options: &DecoderOptions,
+    ) -> Result<Box<dyn Decoder>> {
         if let Some(descriptor) = self.codecs.get(&params.codec) {
             Ok((descriptor.inst_func)(params, options)?)
         }
         else {
-            unsupported_error("unsupported codec")
+            unsupported_error("core (codec):unsupported codec")
         }
     }
 }
 
+impl Default for CodecRegistry {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 /// Convenience macro for declaring a `CodecDescriptor`.
 #[macro_export]
 macro_rules! support_codec {
@@ -413,9 +565,7 @@ macro_rules! support_codec {
             codec: $type,
             short_name: $short_name,
             long_name: $long_name,
-            inst_func: |params, opt| {
-                Ok(Box::new(Self::try_new(&params, &opt)?))
-            }
+            inst_func: |params, opt| Ok(Box::new(Self::try_new(&params, &opt)?)),
         }
     };
-}
\ No newline at end of file
+}
