@@ -1,12 +1,12 @@
 // Symphonia
-// Copyright (c) 2019 The Project Symphonia Developers.
+// Copyright (c) 2019-2022 The Project Symphonia Developers.
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
 //! The `conv` module provides methods to convert samples between different sample types (formats).
-use crate::sample::{Sample, u24, i24};
+use crate::sample::{i24, u24, Sample};
 use crate::util::clamp::*;
 
 pub mod dither {
@@ -26,10 +26,10 @@ pub mod dither {
     //! Multiple dithering algorithms are provided, each drawing noise from a different probability
     //! distribution. In addition to different distributions, a dithering algorithm may also shape
     //! the noise such that the bulk of the noise is placed in an inaudible frequency range.
-    use std::marker::PhantomData;
     use super::FromSample;
-    use crate::sample::{u24, i24};
     use crate::sample::Sample;
+    use crate::sample::{i24, u24};
+    use std::marker::PhantomData;
 
     mod prng {
         #[inline]
@@ -60,9 +60,11 @@ pub mod dither {
 
                 Xoshiro128pp {
                     s: [
-                        (a & 0xffff_ffff) as u32, (a >> 32) as u32,
-                        (b & 0xffff_ffff) as u32, (b >> 32) as u32,
-                    ]
+                        (a & 0xffff_ffff) as u32,
+                        (a >> 32) as u32,
+                        (b & 0xffff_ffff) as u32,
+                        (b >> 32) as u32,
+                    ],
                 }
             }
 
@@ -97,7 +99,7 @@ pub mod dither {
     ///
     /// TODO: `RandomNoise` should be parameterized by the number of bits once const generics land.
     #[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Default)]
-    pub struct RandomNoise (pub i32);
+    pub struct RandomNoise(pub i32);
 
     impl RandomNoise {
         /// Instantiate a noise sample from a random 32-bit source.
@@ -120,10 +122,10 @@ pub mod dither {
         )
     }
 
-    add_noise_impl!(i8 , self, s, {
+    add_noise_impl!(i8, self, s, {
         i8::from_sample(f32::from_sample(s) + f32::from_sample(self.0))
     });
-    add_noise_impl!(u8 , self, s, {
+    add_noise_impl!(u8, self, s, {
         u8::from_sample(f32::from_sample(s) + f32::from_sample(self.0))
     });
     add_noise_impl!(i16, self, s, {
@@ -162,15 +164,20 @@ pub mod dither {
 
     impl<F: Sample, T: Sample> Identity<F, T> {
         pub fn new() -> Self {
-            Identity {
-                from_type: PhantomData,
-                to_type: PhantomData,
-            }
+            Identity { from_type: PhantomData, to_type: PhantomData }
         }
     }
 
     impl<F: Sample, T: Sample> Dither<F, T> for Identity<F, T> {
-        fn dither(&mut self, sample: F) -> F { sample }
+        fn dither(&mut self, sample: F) -> F {
+            sample
+        }
+    }
+
+    impl<F: Sample, T: Sample> Default for Identity<F, T> {
+        fn default() -> Self {
+            Self::new()
+        }
     }
 
     /// `Rectangular` implements a dither using uniformly distributed (white) noise without shaping.
@@ -192,7 +199,7 @@ pub mod dither {
 
     impl<F: Sample, T: Sample> Dither<F, T> for Rectangular<F, T>
     where
-        RandomNoise : AddNoise<F>
+        RandomNoise: AddNoise<F>,
     {
         fn dither(&mut self, sample: F) -> F {
             // A dither should be applied if and only if the effective number of bits of the source
@@ -208,6 +215,12 @@ pub mod dither {
         }
     }
 
+    impl<F: Sample, T: Sample> Default for Rectangular<F, T> {
+        fn default() -> Self {
+            Self::new()
+        }
+    }
+
     /// `Triangular` implements a dither using a triangular distribution of noise without shaping.
     pub struct Triangular<F: Sample, T: Sample> {
         prng: prng::Xoshiro128pp,
@@ -227,7 +240,7 @@ pub mod dither {
 
     impl<F: Sample, T: Sample> Dither<F, T> for Triangular<F, T>
     where
-        RandomNoise : AddNoise<F>
+        RandomNoise: AddNoise<F>,
     {
         fn dither(&mut self, sample: F) -> F {
             debug_assert!(F::EFF_BITS > T::EFF_BITS);
@@ -243,6 +256,12 @@ pub mod dither {
         }
     }
 
+    impl<F: Sample, T: Sample> Default for Triangular<F, T> {
+        fn default() -> Self {
+            Self::new()
+        }
+    }
+
     /// Enumeration of dither algorithms.
     pub enum DitherType {
         /// No dithering.
@@ -255,7 +274,7 @@ pub mod dither {
 
     /// `MaybeDither` conditionally applies a dither to a sample depending on the source and
     /// destination sample types.
-    pub trait MaybeDither<T: Sample> : Sample {
+    pub trait MaybeDither<T: Sample>: Sample {
         const DITHERABLE: bool;
 
         fn maybe_dither<D: Dither<Self, T>>(self, dither: &mut D) -> Self;
@@ -263,7 +282,7 @@ pub mod dither {
 
     /// Never apply a dither for this conversion.
     macro_rules! dither_never {
-        ($to:ty, $from:ty) => (
+        ($to:ty, $from:ty) => {
             impl MaybeDither<$to> for $from {
                 const DITHERABLE: bool = false;
                 #[inline(always)]
@@ -271,12 +290,12 @@ pub mod dither {
                     self
                 }
             }
-        )
+        };
     }
 
     /// Maybe apply a dither for this conversion.
     macro_rules! dither_maybe {
-        ($to:ty, $from:ty) => (
+        ($to:ty, $from:ty) => {
             impl MaybeDither<$to> for $from {
                 const DITHERABLE: bool = true;
                 #[inline(always)]
@@ -284,15 +303,15 @@ pub mod dither {
                     dither.dither(self)
                 }
             }
-        )
+        };
     }
 
     // Dither table for conversions to u8
-    dither_never!(u8, u8 );
+    dither_never!(u8, u8);
     dither_maybe!(u8, u16);
     dither_maybe!(u8, u24);
     dither_maybe!(u8, u32);
-    dither_never!(u8, i8 );
+    dither_never!(u8, i8);
     dither_maybe!(u8, i16);
     dither_maybe!(u8, i24);
     dither_maybe!(u8, i32);
@@ -300,11 +319,11 @@ pub mod dither {
     dither_never!(u8, f64);
 
     // Dither table for conversions to u16
-    dither_never!(u16, u8 );
+    dither_never!(u16, u8);
     dither_never!(u16, u16);
     dither_maybe!(u16, u24);
     dither_maybe!(u16, u32);
-    dither_never!(u16, i8 );
+    dither_never!(u16, i8);
     dither_never!(u16, i16);
     dither_maybe!(u16, i24);
     dither_maybe!(u16, i32);
@@ -312,11 +331,11 @@ pub mod dither {
     dither_never!(u16, f64);
 
     // Dither table for conversions to u24
-    dither_never!(u24, u8 );
+    dither_never!(u24, u8);
     dither_never!(u24, u16);
     dither_never!(u24, u24);
     dither_maybe!(u24, u32);
-    dither_never!(u24, i8 );
+    dither_never!(u24, i8);
     dither_never!(u24, i16);
     dither_never!(u24, i24);
     dither_maybe!(u24, i32);
@@ -324,11 +343,11 @@ pub mod dither {
     dither_never!(u24, f64);
 
     // Dither table for conversions to u32
-    dither_never!(u32, u8 );
+    dither_never!(u32, u8);
     dither_never!(u32, u16);
     dither_never!(u32, u24);
     dither_never!(u32, u32);
-    dither_never!(u32, i8 );
+    dither_never!(u32, i8);
     dither_never!(u32, i16);
     dither_never!(u32, i24);
     dither_never!(u32, i32);
@@ -336,11 +355,11 @@ pub mod dither {
     dither_never!(u32, f64);
 
     // Dither table for conversions to i8
-    dither_never!(i8, u8 );
+    dither_never!(i8, u8);
     dither_maybe!(i8, u16);
     dither_maybe!(i8, u24);
     dither_maybe!(i8, u32);
-    dither_never!(i8, i8 );
+    dither_never!(i8, i8);
     dither_maybe!(i8, i16);
     dither_maybe!(i8, i24);
     dither_maybe!(i8, i32);
@@ -348,11 +367,11 @@ pub mod dither {
     dither_never!(i8, f64);
 
     // Dither table for conversions to i16
-    dither_never!(i16, u8 );
+    dither_never!(i16, u8);
     dither_never!(i16, u16);
     dither_maybe!(i16, u24);
     dither_maybe!(i16, u32);
-    dither_never!(i16, i8 );
+    dither_never!(i16, i8);
     dither_never!(i16, i16);
     dither_maybe!(i16, i24);
     dither_maybe!(i16, i32);
@@ -360,11 +379,11 @@ pub mod dither {
     dither_never!(i16, f64);
 
     // Dither table for conversions to i24
-    dither_never!(i24, u8 );
+    dither_never!(i24, u8);
     dither_never!(i24, u16);
     dither_never!(i24, u24);
     dither_maybe!(i24, u32);
-    dither_never!(i24, i8 );
+    dither_never!(i24, i8);
     dither_never!(i24, i16);
     dither_never!(i24, i24);
     dither_maybe!(i24, i32);
@@ -372,11 +391,11 @@ pub mod dither {
     dither_never!(i24, f64);
 
     // Dither table for conversions to i32
-    dither_never!(i32, u8 );
+    dither_never!(i32, u8);
     dither_never!(i32, u16);
     dither_never!(i32, u24);
     dither_never!(i32, u32);
-    dither_never!(i32, i8 );
+    dither_never!(i32, i8);
     dither_never!(i32, i16);
     dither_never!(i32, i24);
     dither_never!(i32, i32);
@@ -384,11 +403,11 @@ pub mod dither {
     dither_never!(i32, f64);
 
     // Dither table for conversions to f32
-    dither_never!(f32, u8 );
+    dither_never!(f32, u8);
     dither_never!(f32, u16);
     dither_never!(f32, u24);
     dither_never!(f32, u32);
-    dither_never!(f32, i8 );
+    dither_never!(f32, i8);
     dither_never!(f32, i16);
     dither_never!(f32, i24);
     dither_never!(f32, i32);
@@ -396,11 +415,11 @@ pub mod dither {
     dither_never!(f32, f64);
 
     // Dither table for conversions to f64
-    dither_never!(f64, u8 );
+    dither_never!(f64, u8);
     dither_never!(f64, u16);
     dither_never!(f64, u24);
     dither_never!(f64, u32);
-    dither_never!(f64, i8 );
+    dither_never!(f64, i8);
     dither_never!(f64, i16);
     dither_never!(f64, i24);
     dither_never!(f64, i32);
@@ -417,22 +436,22 @@ pub trait FromSample<F> {
 }
 
 macro_rules! converter {
-    ($to:ty, $from:ty, $sample:ident, $func:expr) => (
+    ($to:ty, $from:ty, $sample:ident, $func:expr) => {
         impl FromSample<$from> for $to {
             #[inline]
             fn from_sample($sample: $from) -> Self {
                 $func
             }
         }
-    )
+    };
 }
 
 // Conversions to u8
-converter!(u8, u8 , s, s);
+converter!(u8, u8, s, s);
 converter!(u8, u16, s, (s >> 8) as u8);
 converter!(u8, u24, s, ((s.into_u32() & 0xff_ffff) >> 16) as u8);
 converter!(u8, u32, s, (s >> 24) as u8);
-converter!(u8, i8 , s, (s as u8).wrapping_add(0x80));
+converter!(u8, i8, s, (s as u8).wrapping_add(0x80));
 converter!(u8, i16, s, ((s as u16).wrapping_add(0x8000) >> 8) as u8);
 converter!(u8, i24, s, (((s.into_i32() as u32).wrapping_add(0x80_0000) & 0xff_ffff) >> 16) as u8);
 converter!(u8, i32, s, ((s as u32).wrapping_add(0x8000_0000) >> 24) as u8);
@@ -446,11 +465,11 @@ converter!(u8, f64, s, {
 });
 
 // Conversions to u16
-converter!(u16, u8 , s, u16::from(s) << 8);
+converter!(u16, u8, s, u16::from(s) << 8);
 converter!(u16, u16, s, s);
 converter!(u16, u24, s, ((s.into_u32() & 0xff_ffff) >> 8) as u16);
 converter!(u16, u32, s, (s >> 16) as u16);
-converter!(u16, i8 , s, u16::from((s as u8).wrapping_add(0x80)) << 8);
+converter!(u16, i8, s, u16::from((s as u8).wrapping_add(0x80)) << 8);
 converter!(u16, i16, s, (s as u16).wrapping_add(0x8000));
 converter!(u16, i24, s, (((s.into_i32() as u32).wrapping_add(0x80_0000) & 0xff_ffff) >> 8) as u16);
 converter!(u16, i32, s, ((s as u32).wrapping_add(0x8000_0000) >> 16) as u16);
@@ -464,11 +483,11 @@ converter!(u16, f64, s, {
 });
 
 // Conversions to u24
-converter!(u24, u8 , s, u24::from(u32::from(s) << 16));
+converter!(u24, u8, s, u24::from(u32::from(s) << 16));
 converter!(u24, u16, s, u24::from(u32::from(s) << 8));
 converter!(u24, u24, s, u24::from(s.into_u32() & 0xff_ffff));
 converter!(u24, u32, s, u24::from(s >> 8));
-converter!(u24, i8 , s, u24::from(u32::from((s as u8).wrapping_add(0x80)) << 16));
+converter!(u24, i8, s, u24::from(u32::from((s as u8).wrapping_add(0x80)) << 16));
 converter!(u24, i16, s, u24::from(u32::from((s as u16).wrapping_add(0x8000)) << 8));
 converter!(u24, i24, s, u24::from((s.into_i32() as u32).wrapping_add(0x80_0000) & 0xff_ffff));
 converter!(u24, i32, s, u24::from((s as u32).wrapping_add(0x8000_0000) >> 8));
@@ -482,11 +501,11 @@ converter!(u24, f64, s, {
 });
 
 // Conversions to u32
-converter!(u32, u8 , s, u32::from(s) << 24);
+converter!(u32, u8, s, u32::from(s) << 24);
 converter!(u32, u16, s, u32::from(s) << 16);
 converter!(u32, u24, s, (s.into_u32() & 0xff_ffff) << 8);
 converter!(u32, u32, s, s);
-converter!(u32, i8 , s, u32::from((s as u8).wrapping_add(0x80)) << 24);
+converter!(u32, i8, s, u32::from((s as u8).wrapping_add(0x80)) << 24);
 converter!(u32, i16, s, u32::from((s as u16).wrapping_add(0x8000)) << 16);
 converter!(u32, i24, s, ((s.into_i32() as u32).wrapping_add(0x80_0000) & 0xff_ffff) << 8);
 converter!(u32, i32, s, (s as u32).wrapping_add(0x8000_0000));
@@ -500,11 +519,11 @@ converter!(u32, f64, s, {
 });
 
 // Conversions to i8
-converter!(i8, u8 , s, s.wrapping_add(0x80) as i8);
+converter!(i8, u8, s, s.wrapping_add(0x80) as i8);
 converter!(i8, u16, s, (s.wrapping_add(0x8000) >> 8) as i8);
 converter!(i8, u24, s, (s.into_u32().wrapping_add(0x80_0000) >> 16) as i8);
 converter!(i8, u32, s, (s.wrapping_add(0x8000_0000) >> 24) as i8);
-converter!(i8, i8 , s, s);
+converter!(i8, i8, s, s);
 converter!(i8, i16, s, (s >> 8) as i8);
 converter!(i8, i24, s, ((s.into_i32() & 0xff_ffff) >> 16) as i8);
 converter!(i8, i32, s, (s >> 24) as i8);
@@ -512,11 +531,11 @@ converter!(i8, f32, s, clamp_i8((clamp_f32(s) * 128.0).round() as i16));
 converter!(i8, f64, s, clamp_i8((clamp_f64(s) * 128.0).round() as i16));
 
 // Conversions to i16
-converter!(i16, u8 , s, i16::from(s.wrapping_add(0x80)) << 8);
+converter!(i16, u8, s, i16::from(s.wrapping_add(0x80)) << 8);
 converter!(i16, u16, s, s.wrapping_add(0x8000) as i16);
 converter!(i16, u24, s, (s.into_u32().wrapping_add(0x80_0000) >> 8) as i16);
 converter!(i16, u32, s, (s.wrapping_add(0x8000_0000) >> 16) as i16);
-converter!(i16, i8 , s, i16::from(s) << 8);
+converter!(i16, i8, s, i16::from(s) << 8);
 converter!(i16, i16, s, s);
 converter!(i16, i24, s, ((s.into_i32() & 0xff_ffff) >> 8) as i16);
 converter!(i16, i32, s, (s >> 16) as i16);
@@ -524,11 +543,11 @@ converter!(i16, f32, s, clamp_i16((clamp_f32(s) * 32_768.0).round() as i32));
 converter!(i16, f64, s, clamp_i16((clamp_f64(s) * 32_768.0).round() as i32));
 
 // Conversions to i24
-converter!(i24, u8 , s, i24::from((i32::from(s) - 0x80) << 16));
+converter!(i24, u8, s, i24::from((i32::from(s) - 0x80) << 16));
 converter!(i24, u16, s, i24::from((i32::from(s) - 0x8000) << 8));
 converter!(i24, u24, s, i24::from((s.into_u32() & 0xff_ffff) as i32 - 0x80_0000));
 converter!(i24, u32, s, i24::from((s.wrapping_add(0x8000_0000) as i32) >> 8));
-converter!(i24, i8 , s, i24::from(i32::from(s) << 16));
+converter!(i24, i8, s, i24::from(i32::from(s) << 16));
 converter!(i24, i16, s, i24::from(i32::from(s) << 8));
 converter!(i24, i24, s, s);
 converter!(i24, i32, s, i24::from(s >> 8));
@@ -536,11 +555,11 @@ converter!(i24, f32, s, i24::from(clamp_i24((clamp_f32(s) * 8_388_608.0).round()
 converter!(i24, f64, s, i24::from(clamp_i24((clamp_f64(s) * 8_388_608.0).round() as i32)));
 
 // Conversions to i32
-converter!(i32, u8 , s, (i32::from(s) - 0x80) << 24);
+converter!(i32, u8, s, (i32::from(s) - 0x80) << 24);
 converter!(i32, u16, s, (i32::from(s) - 0x8000) << 16);
 converter!(i32, u24, s, ((s.into_u32() & 0xff_ffff) as i32 - 0x80_0000) << 8);
 converter!(i32, u32, s, s.wrapping_add(0x8000_0000) as i32);
-converter!(i32, i8 , s, i32::from(s) << 24);
+converter!(i32, i8, s, i32::from(s) << 24);
 converter!(i32, i16, s, i32::from(s) << 16);
 converter!(i32, i24, s, (s.into_i32() & 0xff_ffff) << 8);
 converter!(i32, i32, s, s);
@@ -548,11 +567,11 @@ converter!(i32, f32, s, clamp_i32((clamp_f32(s) * 2_147_483_648.0).round() as i6
 converter!(i32, f64, s, clamp_i32((clamp_f64(s) * 2_147_483_648.0).round() as i64));
 
 // Conversions to f32
-converter!(f32, u8 , s, f32::from(i8::from_sample(s)) / 128.0);
+converter!(f32, u8, s, f32::from(i8::from_sample(s)) / 128.0);
 converter!(f32, u16, s, f32::from(i16::from_sample(s)) / 32_768.0);
 converter!(f32, u24, s, (i24::from_sample(s).into_i32() as f32) / 8_388_608.0);
 converter!(f32, u32, s, (i32::from_sample(s) as f32) / 2_147_483_648.0);
-converter!(f32, i8 , s, f32::from(s) / 128.0);
+converter!(f32, i8, s, f32::from(s) / 128.0);
 converter!(f32, i16, s, f32::from(s) / 32_768.0);
 converter!(f32, i24, s, (s.into_i32() as f32) / 8_388_608.0);
 converter!(f32, i32, s, (s as f32) / 2_147_483_648.0);
@@ -560,11 +579,11 @@ converter!(f32, f32, s, s);
 converter!(f32, f64, s, s as f32);
 
 // Conversions to f64
-converter!(f64, u8 , s, f64::from(i8::from_sample(s)) / 128.0);
+converter!(f64, u8, s, f64::from(i8::from_sample(s)) / 128.0);
 converter!(f64, u16, s, f64::from(i16::from_sample(s)) / 32_768.0);
 converter!(f64, u24, s, f64::from(i24::from_sample(s).into_i32()) / 8_388_608.0);
 converter!(f64, u32, s, f64::from(i32::from_sample(s)) / 2_147_483_648.0);
-converter!(f64, i8 , s, f64::from(s) / 128.0);
+converter!(f64, i8, s, f64::from(s) / 128.0);
 converter!(f64, i16, s, f64::from(s) / 32_768.0);
 converter!(f64, i24, s, f64::from(s.into_i32()) / 8_388_608.0);
 converter!(f64, i32, s, f64::from(s) / 2_147_483_648.0);
@@ -588,41 +607,44 @@ impl<F, T: FromSample<F>> IntoSample<T> for F {
 
 /// `ReversibleSample` is a trait that when implemented for `Self`, that `Sample` type implements
 /// reversible conversions between `Self` and the parameterized `Sample` type `S`.
-pub trait ReversibleSample<S> : Sample + FromSample<S> + IntoSample<S> {}
+pub trait ReversibleSample<S>: Sample + FromSample<S> + IntoSample<S> {}
 impl<S, T> ReversibleSample<S> for T where T: Sample + FromSample<S> + IntoSample<S> {}
 
-pub trait ConvertibleSample :
-    Sample +
-        FromSample<i8> +
-        FromSample<u8> +
-        FromSample<i16> +
-        FromSample<u16> +
-        FromSample<i24> +
-        FromSample<u24> +
-        FromSample<i32> +
-        FromSample<u32> +
-        FromSample<f32> +
-        FromSample<f64> {}
+pub trait ConvertibleSample:
+    Sample
+    + FromSample<i8>
+    + FromSample<u8>
+    + FromSample<i16>
+    + FromSample<u16>
+    + FromSample<i24>
+    + FromSample<u24>
+    + FromSample<i32>
+    + FromSample<u32>
+    + FromSample<f32>
+    + FromSample<f64>
+{
+}
 
-impl<S> ConvertibleSample for S
-where
-    S: Sample +
-        FromSample<i8> +
-        FromSample<u8> +
-        FromSample<i16> +
-        FromSample<u16> +
-        FromSample<i24> +
-        FromSample<u24> +
-        FromSample<i32> +
-        FromSample<u32> +
-        FromSample<f32> +
-        FromSample<f64> {}
+impl<S> ConvertibleSample for S where
+    S: Sample
+        + FromSample<i8>
+        + FromSample<u8>
+        + FromSample<i16>
+        + FromSample<u16>
+        + FromSample<i24>
+        + FromSample<u24>
+        + FromSample<i32>
+        + FromSample<u32>
+        + FromSample<f32>
+        + FromSample<f64>
+{
+}
 
 #[cfg(test)]
 mod tests {
-    use std::{u8, i8, u16, i16, u32, i32};
-    use crate::sample::{u24, i24, Sample};
     use super::FromSample;
+    use crate::sample::{i24, u24, Sample};
+    use std::{i16, i32, i8, u16, u32, u8};
 
     #[test]
     fn verify_u8_from_sample() {
@@ -658,12 +680,12 @@ mod tests {
         assert_eq!(u8::from_sample(i32::MID), u8::MID);
         assert_eq!(u8::from_sample(i32::MIN), u8::MIN);
 
-        assert_eq!(u8::from_sample( 1.0f32), u8::MAX);
-        assert_eq!(u8::from_sample(   0f32), u8::MID);
+        assert_eq!(u8::from_sample(1.0f32), u8::MAX);
+        assert_eq!(u8::from_sample(0f32), u8::MID);
         assert_eq!(u8::from_sample(-1.0f32), u8::MIN);
 
-        assert_eq!(u8::from_sample( 1.0f64), u8::MAX);
-        assert_eq!(u8::from_sample(   0f64), u8::MID);
+        assert_eq!(u8::from_sample(1.0f64), u8::MAX);
+        assert_eq!(u8::from_sample(0f64), u8::MID);
         assert_eq!(u8::from_sample(-1.0f64), u8::MIN);
     }
 
@@ -701,12 +723,12 @@ mod tests {
         assert_eq!(u16::from_sample(i32::MID), u16::MID);
         assert_eq!(u16::from_sample(i32::MIN), u16::MIN);
 
-        assert_eq!(u16::from_sample( 1.0f32), u16::MAX);
-        assert_eq!(u16::from_sample(   0f32), u16::MID);
+        assert_eq!(u16::from_sample(1.0f32), u16::MAX);
+        assert_eq!(u16::from_sample(0f32), u16::MID);
         assert_eq!(u16::from_sample(-1.0f32), u16::MIN);
 
-        assert_eq!(u16::from_sample( 1.0f64), u16::MAX);
-        assert_eq!(u16::from_sample(   0f64), u16::MID);
+        assert_eq!(u16::from_sample(1.0f64), u16::MAX);
+        assert_eq!(u16::from_sample(0f64), u16::MID);
         assert_eq!(u16::from_sample(-1.0f64), u16::MIN);
     }
 
@@ -744,12 +766,12 @@ mod tests {
         assert_eq!(u24::from_sample(i32::MID), u24::MID);
         assert_eq!(u24::from_sample(i32::MIN), u24::MIN);
 
-        assert_eq!(u24::from_sample( 1.0f32), u24::MAX);
-        assert_eq!(u24::from_sample(   0f32), u24::MID);
+        assert_eq!(u24::from_sample(1.0f32), u24::MAX);
+        assert_eq!(u24::from_sample(0f32), u24::MID);
         assert_eq!(u24::from_sample(-1.0f32), u24::MIN);
 
-        assert_eq!(u24::from_sample( 1.0f64), u24::MAX);
-        assert_eq!(u24::from_sample(   0f64), u24::MID);
+        assert_eq!(u24::from_sample(1.0f64), u24::MAX);
+        assert_eq!(u24::from_sample(0f64), u24::MID);
         assert_eq!(u24::from_sample(-1.0f64), u24::MIN);
     }
 
@@ -787,16 +809,15 @@ mod tests {
         assert_eq!(u32::from_sample(i32::MID), u32::MID);
         assert_eq!(u32::from_sample(i32::MIN), u32::MIN);
 
-        assert_eq!(u32::from_sample( 1.0f32), u32::MAX);
-        assert_eq!(u32::from_sample(   0f32), u32::MID);
+        assert_eq!(u32::from_sample(1.0f32), u32::MAX);
+        assert_eq!(u32::from_sample(0f32), u32::MID);
         assert_eq!(u32::from_sample(-1.0f32), u32::MIN);
 
-        assert_eq!(u32::from_sample( 1.0f64), u32::MAX);
-        assert_eq!(u32::from_sample(   0f64), u32::MID);
+        assert_eq!(u32::from_sample(1.0f64), u32::MAX);
+        assert_eq!(u32::from_sample(0f64), u32::MID);
         assert_eq!(u32::from_sample(-1.0f64), u32::MIN);
     }
 
-
     #[test]
     fn verify_i8_from_sample() {
         assert_eq!(i8::from_sample(u8::MAX), i8::MAX);
@@ -831,12 +852,12 @@ mod tests {
         assert_eq!(i8::from_sample(i32::MID), i8::MID);
         assert_eq!(i8::from_sample(i32::MIN), i8::MIN);
 
-        assert_eq!(i8::from_sample( 1.0f32), i8::MAX);
-        assert_eq!(i8::from_sample(   0f32), i8::MID);
+        assert_eq!(i8::from_sample(1.0f32), i8::MAX);
+        assert_eq!(i8::from_sample(0f32), i8::MID);
         assert_eq!(i8::from_sample(-1.0f32), i8::MIN);
 
-        assert_eq!(i8::from_sample( 1.0f64), i8::MAX);
-        assert_eq!(i8::from_sample(   0f64), i8::MID);
+        assert_eq!(i8::from_sample(1.0f64), i8::MAX);
+        assert_eq!(i8::from_sample(0f64), i8::MID);
         assert_eq!(i8::from_sample(-1.0f64), i8::MIN);
     }
 
@@ -874,12 +895,12 @@ mod tests {
         assert_eq!(i16::from_sample(i32::MID), i16::MID);
         assert_eq!(i16::from_sample(i32::MIN), i16::MIN);
 
-        assert_eq!(i16::from_sample( 1.0f32), i16::MAX);
-        assert_eq!(i16::from_sample(   0f32), i16::MID);
+        assert_eq!(i16::from_sample(1.0f32), i16::MAX);
+        assert_eq!(i16::from_sample(0f32), i16::MID);
         assert_eq!(i16::from_sample(-1.0f32), i16::MIN);
 
-        assert_eq!(i16::from_sample( 1.0f64), i16::MAX);
-        assert_eq!(i16::from_sample(   0f64), i16::MID);
+        assert_eq!(i16::from_sample(1.0f64), i16::MAX);
+        assert_eq!(i16::from_sample(0f64), i16::MID);
         assert_eq!(i16::from_sample(-1.0f64), i16::MIN);
     }
 
@@ -917,12 +938,12 @@ mod tests {
         assert_eq!(i24::from_sample(i32::MID), i24::MID);
         assert_eq!(i24::from_sample(i32::MIN), i24::MIN);
 
-        assert_eq!(i24::from_sample( 1.0f32), i24::MAX);
-        assert_eq!(i24::from_sample(   0f32), i24::MID);
+        assert_eq!(i24::from_sample(1.0f32), i24::MAX);
+        assert_eq!(i24::from_sample(0f32), i24::MID);
         assert_eq!(i24::from_sample(-1.0f32), i24::MIN);
 
-        assert_eq!(i24::from_sample( 1.0f64), i24::MAX);
-        assert_eq!(i24::from_sample(   0f64), i24::MID);
+        assert_eq!(i24::from_sample(1.0f64), i24::MAX);
+        assert_eq!(i24::from_sample(0f64), i24::MID);
         assert_eq!(i24::from_sample(-1.0f64), i24::MIN);
     }
 
@@ -960,98 +981,98 @@ mod tests {
         assert_eq!(i32::from_sample(i32::MID), i32::MID);
         assert_eq!(i32::from_sample(i32::MIN), i32::MIN);
 
-        assert_eq!(i32::from_sample( 1.0f32), i32::MAX);
-        assert_eq!(i32::from_sample(   0f32), i32::MID);
+        assert_eq!(i32::from_sample(1.0f32), i32::MAX);
+        assert_eq!(i32::from_sample(0f32), i32::MID);
         assert_eq!(i32::from_sample(-1.0f32), i32::MIN);
 
-        assert_eq!(i32::from_sample( 1.0f64), i32::MAX);
-        assert_eq!(i32::from_sample(   0f64), i32::MID);
+        assert_eq!(i32::from_sample(1.0f64), i32::MAX);
+        assert_eq!(i32::from_sample(0f64), i32::MID);
         assert_eq!(i32::from_sample(-1.0f64), i32::MIN);
     }
 
     #[test]
     fn verify_f64_from_sample() {
-        assert_eq!(f64::from_sample(u8::MAX),  127.0 / 128.0);
-        assert_eq!(f64::from_sample(u8::MID),  0.0);
+        assert_eq!(f64::from_sample(u8::MAX), 127.0 / 128.0);
+        assert_eq!(f64::from_sample(u8::MID), 0.0);
         assert_eq!(f64::from_sample(u8::MIN), -1.0);
 
-        assert_eq!(f64::from_sample(u16::MAX),  32_767.0 / 32_768.0);
-        assert_eq!(f64::from_sample(u16::MID),  0.0);
+        assert_eq!(f64::from_sample(u16::MAX), 32_767.0 / 32_768.0);
+        assert_eq!(f64::from_sample(u16::MID), 0.0);
         assert_eq!(f64::from_sample(u16::MIN), -1.0);
 
-        assert_eq!(f64::from_sample(u24::MAX),  8_388_607.0 / 8_388_608.0);
-        assert_eq!(f64::from_sample(u24::MID),  0.0);
+        assert_eq!(f64::from_sample(u24::MAX), 8_388_607.0 / 8_388_608.0);
+        assert_eq!(f64::from_sample(u24::MID), 0.0);
         assert_eq!(f64::from_sample(u24::MIN), -1.0);
 
-        assert_eq!(f64::from_sample(u32::MAX),  2_147_483_647.0 / 2_147_483_648.0);
-        assert_eq!(f64::from_sample(u32::MID),  0.0);
+        assert_eq!(f64::from_sample(u32::MAX), 2_147_483_647.0 / 2_147_483_648.0);
+        assert_eq!(f64::from_sample(u32::MID), 0.0);
         assert_eq!(f64::from_sample(u32::MIN), -1.0);
 
-        assert_eq!(f64::from_sample(i8::MAX),  127.0 / 128.0);
-        assert_eq!(f64::from_sample(i8::MID),  0.0);
+        assert_eq!(f64::from_sample(i8::MAX), 127.0 / 128.0);
+        assert_eq!(f64::from_sample(i8::MID), 0.0);
         assert_eq!(f64::from_sample(i8::MIN), -1.0);
 
-        assert_eq!(f64::from_sample(i16::MAX),  32_767.0 / 32_768.0);
-        assert_eq!(f64::from_sample(i16::MID),  0.0);
+        assert_eq!(f64::from_sample(i16::MAX), 32_767.0 / 32_768.0);
+        assert_eq!(f64::from_sample(i16::MID), 0.0);
         assert_eq!(f64::from_sample(i16::MIN), -1.0);
 
-        assert_eq!(f64::from_sample(i24::MAX),  8_388_607.0 / 8_388_608.0);
-        assert_eq!(f64::from_sample(i24::MID),  0.0);
+        assert_eq!(f64::from_sample(i24::MAX), 8_388_607.0 / 8_388_608.0);
+        assert_eq!(f64::from_sample(i24::MID), 0.0);
         assert_eq!(f64::from_sample(i24::MIN), -1.0);
 
-        assert_eq!(f64::from_sample(i32::MAX),  2_147_483_647.0 / 2_147_483_648.0);
-        assert_eq!(f64::from_sample(i32::MID),  0.0);
+        assert_eq!(f64::from_sample(i32::MAX), 2_147_483_647.0 / 2_147_483_648.0);
+        assert_eq!(f64::from_sample(i32::MID), 0.0);
         assert_eq!(f64::from_sample(i32::MIN), -1.0);
 
-        assert_eq!(f64::from_sample( 1.0f32),  1.0);
-        assert_eq!(f64::from_sample(   0f32),  0.0);
+        assert_eq!(f64::from_sample(1.0f32), 1.0);
+        assert_eq!(f64::from_sample(0f32), 0.0);
         assert_eq!(f64::from_sample(-1.0f32), -1.0);
 
-        assert_eq!(f64::from_sample( 1.0f64),  1.0);
-        assert_eq!(f64::from_sample(   0f64),  0.0);
+        assert_eq!(f64::from_sample(1.0f64), 1.0);
+        assert_eq!(f64::from_sample(0f64), 0.0);
         assert_eq!(f64::from_sample(-1.0f64), -1.0);
     }
 
     #[test]
     fn verify_f32_from_sample() {
-        assert_eq!(f32::from_sample(u8::MAX),  127.0 / 128.0);
-        assert_eq!(f32::from_sample(u8::MID),  0.0);
+        assert_eq!(f32::from_sample(u8::MAX), 127.0 / 128.0);
+        assert_eq!(f32::from_sample(u8::MID), 0.0);
         assert_eq!(f32::from_sample(u8::MIN), -1.0);
 
-        assert_eq!(f32::from_sample(u16::MAX),  32_767.0 / 32_768.0);
-        assert_eq!(f32::from_sample(u16::MID),  0.0);
+        assert_eq!(f32::from_sample(u16::MAX), 32_767.0 / 32_768.0);
+        assert_eq!(f32::from_sample(u16::MID), 0.0);
         assert_eq!(f32::from_sample(u16::MIN), -1.0);
 
-        assert_eq!(f32::from_sample(u24::MAX),  8_388_607.0 / 8_388_608.0);
-        assert_eq!(f32::from_sample(u24::MID),  0.0);
+        assert_eq!(f32::from_sample(u24::MAX), 8_388_607.0 / 8_388_608.0);
+        assert_eq!(f32::from_sample(u24::MID), 0.0);
         assert_eq!(f32::from_sample(u24::MIN), -1.0);
 
-        assert_eq!(f32::from_sample(u32::MAX),  2_147_483_647.0 / 2_147_483_648.0);
-        assert_eq!(f32::from_sample(u32::MID),  0.0);
+        assert_eq!(f32::from_sample(u32::MAX), 2_147_483_647.0 / 2_147_483_648.0);
+        assert_eq!(f32::from_sample(u32::MID), 0.0);
         assert_eq!(f32::from_sample(u32::MIN), -1.0);
 
-        assert_eq!(f32::from_sample(i8::MAX),  127.0 / 128.0);
-        assert_eq!(f32::from_sample(i8::MID),  0.0);
+        assert_eq!(f32::from_sample(i8::MAX), 127.0 / 128.0);
+        assert_eq!(f32::from_sample(i8::MID), 0.0);
         assert_eq!(f32::from_sample(i8::MIN), -1.0);
 
-        assert_eq!(f32::from_sample(i16::MAX),  32_767.0 / 32_768.0);
-        assert_eq!(f32::from_sample(i16::MID),  0.0);
+        assert_eq!(f32::from_sample(i16::MAX), 32_767.0 / 32_768.0);
+        assert_eq!(f32::from_sample(i16::MID), 0.0);
         assert_eq!(f32::from_sample(i16::MIN), -1.0);
 
-        assert_eq!(f32::from_sample(i24::MAX),  8_388_607.0 / 8_388_608.0);
-        assert_eq!(f32::from_sample(i24::MID),  0.0);
+        assert_eq!(f32::from_sample(i24::MAX), 8_388_607.0 / 8_388_608.0);
+        assert_eq!(f32::from_sample(i24::MID), 0.0);
         assert_eq!(f32::from_sample(i24::MIN), -1.0);
 
-        assert_eq!(f32::from_sample(i32::MAX),  2_147_483_647.0 / 2_147_483_648.0);
-        assert_eq!(f32::from_sample(i32::MID),  0.0);
+        assert_eq!(f32::from_sample(i32::MAX), 2_147_483_647.0 / 2_147_483_648.0);
+        assert_eq!(f32::from_sample(i32::MID), 0.0);
         assert_eq!(f32::from_sample(i32::MIN), -1.0);
 
-        assert_eq!(f32::from_sample( 1.0f32),  1.0);
-        assert_eq!(f32::from_sample(   0f32),  0.0);
+        assert_eq!(f32::from_sample(1.0f32), 1.0);
+        assert_eq!(f32::from_sample(0f32), 0.0);
         assert_eq!(f32::from_sample(-1.0f32), -1.0);
 
-        assert_eq!(f32::from_sample( 1.0f64),  1.0);
-        assert_eq!(f32::from_sample(   0f64),  0.0);
+        assert_eq!(f32::from_sample(1.0f64), 1.0);
+        assert_eq!(f32::from_sample(0f64), 0.0);
         assert_eq!(f32::from_sample(-1.0f64), -1.0);
     }
-}
\ No newline at end of file
+}
